package de.ejb.remote.server.security;

import java.io.IOException;
import java.security.Principal;
import java.security.acl.Group;
import java.text.MessageFormat;
import java.util.Map;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;

import org.apache.log4j.Logger;
import org.jboss.security.auth.callback.SecurityAssociationCallback;
import org.jboss.security.auth.spi.LdapExtLoginModule;

public class MyLDAPLogin extends LdapExtLoginModule
{
	private static final Logger LOGGER = Logger.getLogger(MyLDAPLogin.class);

	private MyAppPrincipal customPrincipal = null;


	@Override
	public void initialize(final Subject subject, final CallbackHandler callbackHandler, final Map sharedState, final Map options)
	{
		LOGGER.info(MessageFormat.format("invoke MyLDAPLogin::initialize({0}, {1}, {2}, {3})", subject, callbackHandler, sharedState, options));
		super.initialize(subject, callbackHandler, sharedState, options);
	}


	@Override
	protected Group[] getRoleSets() throws LoginException
	{
		final Group[] groups = super.getRoleSets();
		LOGGER.info(MessageFormat.format("MyLDAPLogin::getRoleSets() returning: {0}", groups));
		return groups;
	}


	@Override
	protected String getUsersPassword() throws LoginException
	{
		final String pass = super.getUsersPassword();
		return pass;
	}


	@Override
	protected boolean validatePassword(final String inputPassword, final String expectedPassword)
	{
		final boolean valid = super.validatePassword(inputPassword, expectedPassword);
		LOGGER.info(MessageFormat.format("MyLDAPLogin::validatePassword({0}, {1}) returning {2}", inputPassword, expectedPassword, valid));
		return valid;
	}


	@Override
	protected Principal getIdentity()
	{
		Principal p = this.customPrincipal;
		if(p == null)
		{
			p = super.getIdentity();
		}

		LOGGER.info(MessageFormat.format("MyLDAPLogin::getIdentity() returning {0}", p));
		return p;
	}


	@Override
	public boolean login() throws LoginException
	{
		LOGGER.info("invoked MyLDAPLogin::login()");

		customPrincipal = getClientPrincipal();

		final boolean loginResult = super.login();

		LOGGER.info(MessageFormat.format("MyLDAPLogin::login() returning {0}", loginResult));
		return loginResult;
	}


	@Override
	public boolean logout() throws LoginException
	{
		LOGGER.info("invoked MyLDAPLogin::logout()");
		final boolean logoutResult = super.logout();
		LOGGER.info(MessageFormat.format("MyLDAPLogin::logout() returning {0}", logoutResult));
		return logoutResult;
	}


	@Override
	protected Object getCredentials()
	{
		final Object cred = super.getCredentials();
		LOGGER.info(MessageFormat.format("MyLDAPLogin::getCredentials() returning {0}", cred));
		return cred;
	}


	@Override
	protected String[] getUsernameAndPassword() throws LoginException
	{
		final String[] userPass = super.getUsernameAndPassword();
		LOGGER.info(MessageFormat.format("MyLDAPLogin::getUsernameAndPassword() returning {0}", userPass));
		return userPass;
	}


	@Override
	protected String createPasswordHash(final String username, final String password, final String digestOption) throws LoginException
	{
		final String hash = super.createPasswordHash(username, password, digestOption);
		LOGGER.info(MessageFormat.format("MyLDAPLogin::createPasswordHash({0}, {1}, {2}) returning {3}", username, password, digestOption));
		return hash;
	}


	@Override
	protected Principal createIdentity(final String username) throws Exception
	{
		final Principal id = super.createIdentity(username);
		LOGGER.info(MessageFormat.format("MyLDAPLogin::createIdentity({0}) returning {1}", username, id));
		return id;
	}


	private MyAppPrincipal getClientPrincipal()
	{
		final SecurityAssociationCallback callback = new SecurityAssociationCallback();
		final Callback[] callbacks =
		{ callback };
		try
		{
			callbackHandler.handle(callbacks);
			final Principal principal = callback.getPrincipal();
			if(principal instanceof MyAppPrincipal)
			{
				return (MyAppPrincipal) principal;
			}
		}
		catch(final IOException e)
		{
			LOGGER.error("couldn't get custom principal", e);
		}
		catch(final UnsupportedCallbackException e)
		{
			LOGGER.error("couldn't get custom principal", e);
		}
		return null;

	}
}
